/*
 * Copyright (C) 2021 B.R.S. Roso.
 * 
 * Xselection is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License version 3 as published by the Free
 * Software Foundation.
 * 
 * This program is distributed in the hope that it be useful but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Xselection.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NB: BEST TO COMPILE THIS WITH ALL OPTIMISATIONS DISABLED. */

/* 
 * Shred tries to prevent the compiler from optimizing away the writting of
 * zeros over a sensitive piece of data. Provided shred gets called as an
 * external function, it should work even with all optimisations enabled due to
 * the use of volatile below. To be sure, I would disable ALL optimisations
 * anyhow. 
 */

#include <stddef.h>

/* Might be a good idea to write random data instead of zeros, this would it
 * make it difficult to determine the length of the data we erase, but I
 * figured that, at least in the use I had in mind when I wrote this, the
 * memory would be allocated in blocks larger than the data anyway, so it's not
 * worth the effort. */
void shred(volatile void *p, size_t n)
{
	volatile char *q = p;
	while (n--)
		*q++ = 0;
}
